// sntpc.c  - simple NTP client

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <linux/types.h>

#include <sys/types.h>
#include <netdb.h>

#ifdef TI_RTOS
# include "ti_rtos_io.h"
# include "NDK/netmain.h"
//# include "NDK/_stack.h"
# include "NDK/socketndk.h"
# include "NDK/usertype.h"
# include "NDK/socket.h"
# include "NDK/nettools/nettools.h"
# include <common.h>	// u-boot header

#ifdef CONFIG_CMD_SNTP
# include <net.h>	// u-boot header
# include <rtc.h>	// u-boot header
# define htons NDK_htons
# define htonl NDK_htonl
# define ntohs NDK_ntohs
# define ntohl NDK_ntohl
# define closesocket fdClose
# define select fdSelect
# define FD_ISSET	NDK_FD_ISSET
# define FD_SET	NDK_FD_SET
# define FD_ZERO	NDK_FD_ZERO
# define fd_set	NDK_fd_set
# define fprintf(fp,msg)	printf("%s",msg)
#else //-TI_RTOS
# include <sys/socket.h>
# include <netinet/in.h>
  typedef int SOCKET;
# define closesocket close
# define SOCKET_ERROR (-1)
# define INVALID_SOCKET (-1)
#endif //-TI_RTOS

typedef long long int __int64;
#define TRUE 1      //true
#define FALSE 0     //false
#define BOOL int
#define stricmp strcasecmp


#define BUFSIZE		256
#define TIMEOUT		3	    // 3s

struct NTP_Packet{
    int Control_Word;
    int root_delay;
    int root_dispersion;
    int reference_identifier;
    __int64 reference_timestamp;
    __int64 originate_timestamp;
    __int64 receive_timestamp;
    int transmit_timestamp_seconds;
    int transmit_timestamp_fractions;
};


#include <ti/net/sntp/sntp.h>
static int _connect_sntp_server(const char* serverName)
{
    SlNetSock_Timeval_t timeout={ 3,0 };
    uint64_t ntpTimeStamp=0;
    int sts=0;
    uint32_t serveraddr = inet_addr((char*)serverName);
    if(serveraddr == (unsigned long)-1 || serveraddr==0) {
        // servername
        const char* serverList[2]={ serverName, };
        sts=SNTP_getTime(serverList,1,&timeout,&ntpTimeStamp);
    }
    else {
        // ipaddr
        SlNetSock_AddrIn_t sa4={ 0, };
        sa4.sin_family=SLNETSOCK_AF_INET;
        sa4.sin_port=SlNetUtil_htons(123);
        sa4.sin_addr.s_addr=serveraddr; //SlNetUtil_htonl(serveraddr);
        sts=SNTP_getTimeByAddr((SlNetSock_Addr_t*)&sa4,&timeout,&ntpTimeStamp);
    }
    if (sts) { return sts; }
    unsigned long ntp_time=ntpTimeStamp>>32;
    ntp_time -= 2208988800UL;		/* 1970/01/01 ̕bɕϊ */
    struct rtc_time tm;
    rtc_to_tm(ntp_time + net_ntp_time_offset, &tm);
    printf("Date: %4d-%02d-%02d Time: %2d:%02d:%02d UTC\n",
           tm.tm_year, tm.tm_mon, tm.tm_mday,
           tm.tm_hour, tm.tm_min, tm.tm_sec);
    return 0;
}

int connect_sntp_server(const char* serverName)
{
    if (serverName==NULL) {
        serverName="ntp.nict.jp";
    }
#ifdef TI_RTOS
    // very important
    fdOpenSession(TaskSelf());
    //
    //sleep(5);
    int sts=_connect_sntp_server(serverName);
    // very important too.
    fdCloseSession(TaskSelf());
    //
    return sts;
#else //TI_RTOS
    return _connect_sntp_server(serverName);
#endif //TI_RTOS
}

#endif //CONFIG_CMD_SNTP
