/*
 * ti_rtos_io.c
 *
 *  Created on: 2018/11/19
 *      Author: miyake
 */
/* Standard header files */
#include <stdbool.h>
//#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#include <mqueue.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>

// cannot use for bug.
//#include <ti/devices/msp432e4/driverlib/rom.h>

/* Driver header files */
//#include <ti/drivers/UART.h>

/* Example/Board Header files */
#include "Board_ubootcmd.h"
#include "board_funcs.h"
#include "cloud_task.h"

#include "ti_rtos_io.h"
#include "USB/USBMSCH.h"

//============================
// u-boot helper functions
//----------------------------
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/drivers/Watchdog.h>
#include <ti/devices/msp432e4/inc/msp432e401y.h>	// cpu reg addr defines.
#include <ti/devices/msp432e4/driverlib/gpio.h>	// gpio device-driver API
#include <xdc/runtime/Types.h>
//----------------------------
// u-boot header
//----------------------------
#include "config.h"	// u-boot header
#include "status_led.h"	// u-boot header
#include "command.h"	// u-boot header
#include "vsprintf.h"	// u-boot header
//#include "linux/errno.h"	// u-boot header
#include "dm/uclass-id.h"	// u-boot header
#include "asm/global_data.h"	// u-boot header
// u-boot board information global
#include <Board.h>
static bd_t bd_buf=
{
	 // no DRAM
	.bi_memstart=0,	/* start of DRAM memory */
	.bi_memsize=0,	/* size	 of DRAM memory in bytes */
	// has Flash
	.bi_flashstart=CONFIG_SYS_FLASH_BASE,	/* start of FLASH memory */
	.bi_flashsize=CONFIG_SYS_FLASH_SIZE,	/* size	 of FLASH memory */
	//unsigned long	bi_flashoffset; /* reserved area for startup monitor */
	// has SRAM
	.bi_sramstart=CONFIG_SYS_SRAM_BASE,	/* start of SRAM memory */
	.bi_sramsize=CONFIG_SYS_SRAM_SIZE,	/* size	 of SRAM memory */

};
gd_t gd_buf={
	.bd=&bd_buf,
	.flags=GD_FLG_RELOC,	// for HUSH history (command line edit)
	// cannot. so set in uboot_init()
	//.cpu_clk=((Types_FreqHz)BIOS_cpuFreq).lo,		/* CPU clock in Hz!		*/
	// baudreate
	.baudrate=CONFIG_BAUDRATE,
};
ulong load_addr = CONFIG_SYS_LOAD_ADDR;	/* Default Load Address */
ulong save_addr;			/* Default Save Address */
ulong save_size;			/* Default Save Size (in bytes) */
//----------------------------
//============================


//============================
// initialize for u-boot
//----------------------------
void uboot_init(void) {
	// cpuFreq
	extern const Types_FreqHz BIOS_cpuFreq;
	gd_buf.cpu_clk=BIOS_cpuFreq.lo;
}
//============================

//============================
// cache
//----------------------------
void	enable_caches(void) {};
void	flush_cache   (unsigned long a, unsigned long b) {};
void	flush_dcache_all(void) {};
void	flush_dcache_range(unsigned long start, unsigned long stop) {};
void	invalidate_dcache_range(unsigned long start, unsigned long stop) {};
void	invalidate_dcache_all(void) {};
void	invalidate_icache_all(void) {};
//============================

//============================
// timer
//----------------------------
unsigned long timer_read_counter(void)
{
	return Clock_getTicks();
}
//============================


#include "uart6.h"
//============================
// UART (UART6 as console)
//----------------------------
# ifdef USE_UART6
#include "command_task.h"
/* An array to hold the data to be displayed on UART console. */
char g_pcTXBuf[TX_BUF_SIZE];
//----------------------------
// program -> output to console
int ti_rtos_putc(const int c)
{
	return putc_uart6_tx_buf(c);
}
int ti_rtos_puts(const char* s)
{
	return puts_uart6_tx_buf(s);
}
int ti_rtos_printf(const char* fmt, ...)
{
    //uint32_t ui32BufLen;
    va_list ap;

	va_start(ap, fmt);
	/*ui32BufLen =*/ vsnprintf(g_pcTXBuf, TX_BUF_SIZE, fmt, ap);
	va_end(ap);

	return puts_uart6_tx_buf(g_pcTXBuf);
}
//----------------------------
// program <- input from console
int ti_rtos_getc(void)
{
	int c=getc_uart6_rx_buf();
	if (c<0) {
        return CMDLINE_UART_ERROR;
    }
    return c;
}
int ti_rtos_tstc(void)
{
	return (! is_uart6_rx_buf_empty());
}
static int ctrlc_disabled=0;
static int ctrlc_was_pressed=0;
int ctrlc(void)
{
	if (ctrlc_disabled) { return 0; }
	if (ti_rtos_tstc()) {
		// key inputted -> check ^C
		if (ti_rtos_getc()==0x03) {
			ctrlc_was_pressed=1;
			return 1;
		}
	}
	return 0;
}
int had_ctrlc(void)	/* have we had a Control-C since last clear? */
{
	return ctrlc_was_pressed;
}
void clear_ctrlc(void)	/* clear the Control-C condition */
{
	ctrlc_was_pressed=0;
}
int disable_ctrlc(int disable)	/* 1 to disable, 0 to enable Control-C detect */
{
	int prev=ctrlc_disabled;
	ctrlc_disabled=disable;
	return prev;
}
# endif //USE_UART6
//============================


//============================
// LED (GPIO)
//----------------------------
#ifdef CONFIG_LED_STATUS
#include <ti/drivers/GPIO.h>
static inline int get_led_gpio(const led_id_t mask) {
	if (CONFIG_LED_STATUS_BIT & mask) { return Board_GPIO_LED0; }
	if (CONFIG_LED_STATUS_BIT1 & mask) { return Board_GPIO_LED1; }
	return 0;
}
/*
void __led_init(led_id_t mask, int state)
{
	// GPIO_init();
}*/
void __led_toggle(led_id_t mask)
{
	int board_gpio_led=get_led_gpio(mask);
	if (board_gpio_led) {
		// set out and state
        GPIO_toggle(board_gpio_led);
	}
}
void __led_set(led_id_t mask, int state)
{
	int board_gpio_led=get_led_gpio(mask);
	if (board_gpio_led) {
		// set out and state
        GPIO_write(board_gpio_led, state);
	}
}
void __led_blink(led_id_t mask, int freq)
{
	int board_gpio_led=get_led_gpio(mask);
	if (board_gpio_led) {
		puts("blink is not supported.\n");
	}
}
#endif //CONFIG_LED_STATUS
//============================

//============================
// GPIO
//----------------------------
#ifdef CONFIG_CMD_GPIO
#include <ti/drivers/GPIO.h>
// Port A=validGPIOPins[0], B=validGPIOPins[1],...
static int _validGPIOPortPins[]=
{
#ifdef Board_MSP_EXP432E401Y
	// A,    B,    C,    D,    E,    F,    G,    H,    I,    J,    K,    L,    M,    N,    O,    P,    Q,
	// 0,    1,    2,    3,    4,    5,    6,    7,    8,    9,   10,   11,   12,   13,   14,   15,   16,
	0xFF, 0x3F, 0xFF, 0xFF, 0x3F, 0x1F, 0x03, 0x0F, 0x00, 0x03, 0xFF, 0xFF, 0xFF, 0x3F, 0x00, 0x3F, 0x1F,
#endif
};
static const int _validGPIOPortPinsCount=sizeof(_validGPIOPortPins)/sizeof(_validGPIOPortPins[0]);	// =17
static uint32_t gpioPortAddr[]=
{
	 GPIO_PORTA_BASE, GPIO_PORTB_BASE, GPIO_PORTC_BASE, GPIO_PORTD_BASE, GPIO_PORTE_BASE,
	 GPIO_PORTF_BASE, GPIO_PORTG_BASE, GPIO_PORTH_BASE, NULL,            GPIO_PORTJ_BASE,
	 GPIO_PORTK_BASE, GPIO_PORTL_BASE, GPIO_PORTM_BASE, GPIO_PORTN_BASE, NULL,
	 GPIO_PORTP_BASE, GPIO_PORTQ_BASE,
};
static inline int getGPIOPort(const int gpio) { return ((gpio)/8); }
static inline int getGPIOPin(const int gpio) { return ((gpio)&7); }
// return PA0->0, PA1->1, ..., PB0->8, ..., PK0->80, ...
int name_to_gpio(const char *name)
{
	if (isdigit(name[0])) {
		// name is portNumber (digit)
		return simple_strtoul(name, NULL, 10);
	}
	// name is 'PAn'
	int c=toupper(name[0]);
	if (c!='P') { return -1; }
	c=toupper(name[1]);
	int port=c-'A';
	if (0<=port && port<_validGPIOPortPinsCount) {
		// ok
	}
	else {
		// invalid
		return -1;
	}
	// number
	if (! isdigit(name[2])) { return -1; }
	int pin=name[2]-'0';
	if (0<=pin && pin<8) {
		// ok
	}
	else {
		// invalid
		return -1;
	}
	// check existence of pin
	if (_validGPIOPortPins[port]&(1<<pin)) {
		// exist
		return (port*8+pin);
	}
	// invalid
	return -1;
}
int isValidGPIOPin(const int gpioPinNum) {
	if (gpioPinNum<0 || _validGPIOPortPinsCount*8<=gpioPinNum) {
		// invalid
		return 0;
	}
	// check existence
	if (_validGPIOPortPins[gpioPinNum/8]&(1<<(gpioPinNum&7))) {
		// exist
		return 1;
	}
	return 0;
}
/**
 * @deprecated	Please use driver model instead
 * Request a GPIO. This should be called before any of the other functions
 * are used on this GPIO.
 *
 * Note: With driver model, the label is allocated so there is no need for
 * the caller to preserve it.
 *
 * @param gpio	GPIO number
 * @param label	User label for this GPIO
 * @return 0 if ok, -1 on error
 */
int gpio_request(unsigned gpio, const char *label)
{
	return (isValidGPIOPin(gpio)?0:-1);
}
/**
 * @deprecated	Please use driver model instead
 * Stop using the GPIO.  This function should not alter pin configuration.
 *
 * @param gpio	GPIO number
 * @return 0 if ok, -1 on error
 */
int gpio_free(unsigned gpio)
{
	return (isValidGPIOPin(gpio)?0:-1);
}
/**
 * @deprecated	Please use driver model instead
 * Make a GPIO an input.
 *
 * @param gpio	GPIO number
 * @return 0 if ok, -1 on error
 */
int gpio_direction_input(unsigned gpio)
{
	if (! isValidGPIOPin(gpio)) { return -1; }
	// cannot use RTOS SDK driver library. it use GPIO_PinConfig, it must set BEFORE GPIO_init().

	int port=getGPIOPort(gpio);
	int pin=getGPIOPin(gpio);
	int pinBit=1<<pin;
	uint32_t addr= gpioPortAddr[port];
	if (addr==NULL) { return -1; }
	//
	//GPIODirModeSet(addr,pinBit,GPIO_DIR_MODE_IN);
	//
	// keep strength,pu/pd
	uint32_t strength=0;
	uint32_t pintype=0;
	GPIOPadConfigGet(addr,pin,&strength,&pintype);	// note: not pinBits, it it pin
	//
	GPIOPinTypeGPIOInput(addr,pinBit);
	// if setting modification. ex)pullup
	if (pintype==GPIO_PIN_TYPE_STD_WPU||pintype==GPIO_PIN_TYPE_STD_WPD) {
		// keep it
	}
	else {
		pintype=GPIO_PIN_TYPE_STD;
	}
	// revive strength,pu/pd
	GPIOPadConfigSet(addr,pinBit,strength,pintype);
	//
	return 0;
}
/**
 * @deprecated	Please use driver model instead
 * Make a GPIO an output, and set its value.
 *
 * @param gpio	GPIO number
 * @param value	GPIO value (0 for low or 1 for high)
 * @return 0 if ok, -1 on error
 */
int gpio_direction_output(unsigned gpio, int value)
{
	if (! isValidGPIOPin(gpio)) { return -1; }
	// cannot use RTOS SDK driver library. it use GPIO_PinConfig, it must set BEFORE GPIO_init().

	int port=getGPIOPort(gpio);
	int pin=getGPIOPin(gpio);
	int pinBit=1<<pin;
	uint32_t addr= gpioPortAddr[port];
	if (addr==NULL) { return -1; }
	//
	//GPIODirModeSet(addr,pinBit,GPIO_DIR_MODE_OUT);
	//
	// keep strength,pu/pd
	uint32_t strength=0;
	uint32_t pintype=0;
	GPIOPadConfigGet(addr,pin,&strength,&pintype);	// note: not pinBits, it it pin
	//
	GPIOPinTypeGPIOOutput(addr,pinBit);
	// if setting modification. ex)pullup
	if (pintype==GPIO_PIN_TYPE_STD_WPU||pintype==GPIO_PIN_TYPE_STD_WPD) {
		// keep it
	}
	else {
		pintype=GPIO_PIN_TYPE_STD;
	}
	// revive strength,pu/pd
	GPIOPadConfigSet(addr,pinBit,strength,pintype);
	//
	return 0;
}
/**
 * @deprecated	Please use driver model instead
 * Get a GPIO's value. This will work whether the GPIO is an input
 * or an output.
 *
 * @param gpio	GPIO number
 * @return 0 if low, 1 if high, -1 on error
 */
int gpio_get_value(unsigned gpio)
{
	if (! isValidGPIOPin(gpio)) { return -1; }
	// cannot use RTOS SDK driver library. it use GPIO_PinConfig, it must set BEFORE GPIO_init().

	int port=getGPIOPort(gpio);
	int pin=getGPIOPin(gpio);
	int pinBit=1<<pin;
	uint32_t addr= gpioPortAddr[port];
	if (addr==NULL) { return -1; }
	if (GPIOPinRead(addr,pinBit)) {
		// on (note: GPIOPinRead() returns (pinBits & *dataReg))
		return 1;
	}
	// off
	return 0;
}
/**
 * @deprecated	Please use driver model instead
 * Set an output GPIO's value. The GPIO must already be an output or
 * this function may have no effect.
 *
 * @param gpio	GPIO number
 * @param value	GPIO value (0 for low or 1 for high)
 * @return 0 if ok, -1 on error
 */
int gpio_set_value(unsigned gpio, int value)
{
	if (! isValidGPIOPin(gpio)) { return -1; }
	// cannot use RTOS SDK driver library. it use GPIO_PinConfig, it must set BEFORE GPIO_init().

	int port=getGPIOPort(gpio);
	int pin=getGPIOPin(gpio);
	int pinBit=1<<pin;
	uint32_t addr= gpioPortAddr[port];
	if (addr==NULL) { return -1; }
	GPIOPinWrite(addr,pinBit,value?pinBit:0);
	// on (note: GPIOPinWrite() set *dataReg=(*dataReg& ~pinBits)|(data&pinBits))
	return 0;
}
#endif //CONFIG_CMD_GPIO
//============================


//============================
// reset (using WatchDog)
//----------------------------
static void UserCallbackFxn(uintptr_t handle) {}
//----------------------------
int do_reset(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	Watchdog_Params params;
	Watchdog_init();
	Watchdog_Params_init(&params);
	params.resetMode = Watchdog_RESET_ON;
	params.callbackFxn = (Watchdog_Callback) UserCallbackFxn;

	Watchdog_Handle watchdogHandle;
	watchdogHandle = Watchdog_open(Board_WATCHDOG0, &params);
    if (watchdogHandle == NULL) {
        // Error opening Watchdog
        while (1);
    }
    puts("resetting...");
	//WatchdogResetEnable(watchdogHandle);
	const int ms=3*1000;	//3s
	for (;;) { Task_sleep(ms); }
}
//----------------------------
int do_poweroff(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	// not support
	return -1;
}
//============================

//============================
// EEPROM (on MSP432)
//----------------------------
#ifdef CONFIG_SYS_MSP432_EEPROM	//CONFIG_CMD_EEPROM
//----------------------------
#include <ti/devices/msp432e4/driverlib/eeprom.h>
//----------------------------
//#include "i2c.h"	// u-boot header
//----------------------------
int onchip_eeprom_init(void)
{
	static int inited=0;
	if (inited) { return 0; }
	int sts=EEPROMInit();
	if (sts==EEPROM_INIT_OK) {
		inited=1;
		return 0;
	}
	return -1;
}
int  onchip_eeprom_read(void* buffer, unsigned int offset, unsigned int len)
{
	// must be 4byte align, length must be multiple of 4
	if (((int)buffer&3) || (offset&3) || (len&3)) {
		return -1;
	}
	EEPROMRead((uint32_t*)buffer,offset,len);
	return 0;
}
int  onchip_eeprom_write(void* buffer, unsigned int offset, unsigned int len)
{
	// must be 4byte align, length must be multiple of 4
	if (((int)buffer&3) || (offset&3) || (len&3)) {
		return -1;
	}
	return EEPROMProgram((uint32_t*)buffer,offset,len);
}
int onchip_eeprom_size(void)
{
	onchip_eeprom_init();
	return EEPROMSizeGet();
}
#endif //CONFIG_SYS_MSP432_EEPROM	//CONFIG_CMD_EEPROM
//============================

//============================
// Flash (on MSP432)
//----------------------------
#ifdef CONFIG_SYS_MSP432_FLASH	//CONFIG_CMD_FLASH
//----------------------------
#include <ti/devices/msp432e4/driverlib/flash.h>
tFlashProtection bugfixedFlashProtectGet(uint32_t ui32Address);
int32_t bugfixedFlashProtectSet(uint32_t ui32Address, tFlashProtection eProtect);
//#define bugfixedFlashProtectGet FlashProtectGet
//#define bugfixedFlashProtectSet FlashProtectSet
#ifdef ROM_FlashProtectSave
# define FlashProtectSave ROM_FlashProtectSave
#endif //ROM_FlashProtectSave
//----------------------------
//#include <common.h>	// u-boot header.
typedef unsigned char		uchar;
#include <flash.h>	// u-boot header.
/* info for FLASH chips	*/
#define ONCHIP_FLASH_SECT_SIZE	(16*1024)
flash_info_t flash_info[CONFIG_SYS_MAX_FLASH_BANKS]={
	{
		//ulong	size;			/* total bank size in bytes		*/
		.size= 1024*1024,		// 1MB flash
		//ushort	sector_count;		/* number of erase units		*/
		.sector_count=1024/16,	// 16KB/sector
		//ulong	flash_id;		/* combined device & manufacturer code	*/
		.flash_id= 0xFFAA, //not FLASH_UNKNOWN, //0xFFFE,	// as msp432 flash
		//ulong	start[CONFIG_SYS_MAX_FLASH_SECT];   /* virtual sector start address */
		.start={
				0*(16*1024),  1*(16*1024),  2*(16*1024),  3*(16*1024),  4*(16*1024),  5*(16*1024),  6*(16*1024),  7*(16*1024),
				8*(16*1024),  9*(16*1024),  10*(16*1024), 11*(16*1024), 12*(16*1024), 13*(16*1024), 14*(16*1024), 15*(16*1024),
				16*(16*1024), 17*(16*1024), 18*(16*1024), 19*(16*1024), 20*(16*1024), 21*(16*1024), 22*(16*1024), 23*(16*1024),
				24*(16*1024), 25*(16*1024), 26*(16*1024), 27*(16*1024), 28*(16*1024), 29*(16*1024), 30*(16*1024), 31*(16*1024),
				32*(16*1024), 33*(16*1024), 34*(16*1024), 35*(16*1024), 36*(16*1024), 37*(16*1024), 38*(16*1024), 39*(16*1024),
				40*(16*1024), 41*(16*1024), 42*(16*1024), 43*(16*1024), 44*(16*1024), 45*(16*1024), 46*(16*1024), 47*(16*1024),
				48*(16*1024), 49*(16*1024), 50*(16*1024), 51*(16*1024), 52*(16*1024), 53*(16*1024), 54*(16*1024), 55*(16*1024),
				56*(16*1024), 57*(16*1024), 58*(16*1024), 59*(16*1024), 60*(16*1024), 61*(16*1024), 62*(16*1024), 63*(16*1024),
		},
		//uchar	protect[CONFIG_SYS_MAX_FLASH_SECT]; /* sector protection status	*/
		.protect={ 0, },
	},
};
//----------------------------
int onchip_flash_init(void)
{
	static int inited=0;
	if (inited) { return 0; }
	int err=0;

	// readin protect info
	int i;
	for (i=0; i<flash_info[0].sector_count; i++) {
		tFlashProtection eProtect=bugfixedFlashProtectGet(flash_info[0].start[i]);
		int sts=eProtect;
		if (sts<0) {
			// error -> treat as read only
			err++;
			sts=1;
		}
		else if (sts==FlashReadWrite) {
			sts=0;
		}
		else {
			sts=1;
		}
		flash_info[0].protect[i]=sts;
	}
	//
	inited=1;
	return -err;
}
unsigned long flash_init(void)
{
	onchip_flash_init();
	return 0;
}
void flash_print_info (flash_info_t * info)
{
	if (info!= &flash_info[0]) {
		// not onchip flash
		printf("not supported chip. onchip flash only.\n");
		return ;
	}
	printf("OnChip Flash, %d KB/sector, %d sectors(%dKB)\n",
			ONCHIP_FLASH_SECT_SIZE/1024, info->sector_count, info->size/1024);
	int i;
	printf("prot:              11111 11111 22222 22222\n");
	printf("prot:  01234 56789 01234 56789 01234 56789\n");
	for (i=0; i<info->sector_count; i++) {
		//if ((i&0x1F)==0) { printf("%06X ",i*ONCHIP_FLASH_SECT_SIZE); }
		if ((i%30)==0) { printf("%06X ",i*ONCHIP_FLASH_SECT_SIZE); }
		printf(info->protect[i]?"p":".");
		//if ((i&0x1F)==0x1F) { printf("\n"); }
		//else if ((i&0x7)==7) { printf(" "); }
		if ((i%30)==29) { printf("\n"); }
		else if ((i%5)==4) { printf(" "); }
	}
	printf("\n");
}
int flash_erase	(flash_info_t * info, int sect_first, int sect_last)
{
	if (info!= &flash_info[0]) {
		// not onchip flash
		printf("not supported chip. onchip flash only.\n");
		return ERR_UNKNOWN_FLASH_VENDOR;
	}
	if (sect_first<0||info->sector_count<=sect_first ||
		sect_last<0||info->sector_count<=sect_last) {
		// invalid sect
		printf("invalid sector number\n");
		return ERR_INVAL;
	}
	int i;
	for (i=sect_first; i<=sect_last; i++) {
		uint32_t addr=info->start[i];
		printf("... Erasing %d (%x-%x) ... ",i,addr,addr+ONCHIP_FLASH_SECT_SIZE-1);
		int sts=FlashErase(addr);
		if (sts) {
			// error
			printf("ERROR. erase aborted.\n");
			return ERR_NOT_ERASED;
		}
		printf("done.\n");
	}
	return ERR_OK;
}
int write_buff(flash_info_t *info, uchar *src, ulong addr, ulong cnt)
{
	if (info!= &flash_info[0]) {
		// not onchip flash
		printf("not supported chip. onchip flash only.\n");
		return ERR_UNKNOWN_FLASH_VENDOR;
	}
	const int sect_size=ONCHIP_FLASH_SECT_SIZE;
	const int sect_count=info->sector_count;
	if (addr>=sect_size*sect_count) {
		// out of flash
		printf("invalid addr\n");
		return ERR_INVAL;
	}
	if ((addr&3)||(((int)src)&3)) {
		// not aligned
		printf("src/addr is not aligned by 4.\n");
		return ERR_ALIGN;
	}
	int sts=FlashProgram((uint32_t*)src,addr,cnt);
	if (sts) {
		printf("ERROR: not programed. not erased?\n");
		return ERR_PROG_ERROR;
	}
	return ERR_OK;
}
#ifdef CONFIG_SYS_FLASH_PROTECTION
int flash_real_protect(flash_info_t *info, long sector, int prot)
{
	if (info!= &flash_info[0]) {
		// not onchip flash
		printf("not supported chip. onchip flash only.\n");
		return -1;
	}
	if (sector<0||info->sector_count<=sector) {
		// invalid sect
		printf("invalid sector number\n");
		return ERR_INVAL;
	}
	uint32_t addr=info->start[sector];
	tFlashProtection eProtect= prot?FlashReadOnly:FlashReadWrite;
#ifdef DRY_RUN_TO_PROTECT_FLASH
	printf("DRYRUN: setting %sprotect %d sector(%08x-%08x)\n",prot?"":"un",sector,addr,addr+ONCHIP_FLASH_SECT_SIZE-1);
	return ERR_OK;
#else //-DRY_RUN_TO_PROTECT_FLASH
	int sts=bugfixedFlashProtectSet(addr,eProtect);
	if (sts==0) {
		// maintain protect in flash_info
		flash_info[0].protect[sector]=prot;
# if 0
		// save to flash
		sts=FlashProtectSave();
		if (sts) {
			printf("ERROR: cannot set un/protect to flash\n");
		}
# endif //0
	}
	return sts;
#endif //-DRY_RUN_TO_PROTECT_FLASH
}
int flash_real_protect_commit(flash_info_t *info)
{
	if (info!= &flash_info[0]) {
		// not onchip flash
		printf("not supported chip. onchip flash only.\n");
		return -1;
	}
	// save to flash
	int sts=FlashProtectSave();
	if (sts) {
		printf("ERROR: cannot set un/protect to flash\n");
	}
	return sts;
}
#endif //CONFIG_SYS_FLASH_PROTECTION

#endif //CONFIG_SYS_MSP432_FLASH	//CONFIG_CMD_FLASH
//============================


//============================
// I2C
//----------------------------
#ifdef CONFIG_CMD_I2C
//----------------------------
#include <ti/drivers/I2C.h>
//----------------------------
#include "i2c.h"	// u-boot header
//----------------------------
static int cur_i2c_bus=I2C_2; //I2C_0;
static I2C_BitRate cur_i2c_speed=I2C_100kHz;
static const unsigned int board_i2c_speed[]=
{
 // I2C_100kHz, I2C_400kHz, I2C_1000kHz, I2C_3330kHz(I2C_3400kHz)
 100*1000U, 400*1000U, 1000*1000U, 3330*1000U
};
static const int board_i2c_speed_count=sizeof(board_i2c_speed)/sizeof(board_i2c_speed[0]);
static const signed char board_i2c_bus[]={
    //    i2c0             1           2             3   4   5   6   7   8   9  10
    MSP_EXP432E401Y_I2C0, -1, MSP_EXP432E401Y_I2C2, -1, -1, -1, -1, -1, -1, -1, -1,
};
static const int board_i2c_bus_count=sizeof(board_i2c_bus)/sizeof(board_i2c_bus[0]);
//----------------------------
//struct i2c_adapter *i2c_get_adapter(int index);
void i2c_init(int speed, int slaveaddr)
{
	// slaveaddr is BOARD's i2c device's addr
	i2c_set_bus_speed(speed);
}
unsigned int i2c_get_bus_num(void)
{
	return cur_i2c_bus;
}
int i2c_set_bus_num(unsigned int bus)
{
	if (bus>=board_i2c_bus_count) {
		// not such bus
		return -1;
	}
	if (board_i2c_bus[bus]>=0) {
		// ok
		cur_i2c_bus=bus;
		return 0;
	}
	// not exist such bus on board
	return -1;
}
int i2c_probe(uint8_t chip)
{
	//return i2c_read(chip,0,0,NULL,0);
	char buf[2];
	return i2c_read(chip,0,0,buf,1);
}
int i2c_read(uint8_t chip, unsigned int addr, int alen, uint8_t *buffer, int len)
{
	int ret=0;
	I2C_Handle i2cHandle=NULL;
	I2C_Params i2cParams;
	I2C_Params_init(&i2cParams);
	i2cParams.transferMode = I2C_MODE_BLOCKING;
	//i2cParams.transferCallbackFxn = myCallbackFunction;
	i2cParams.bitRate = cur_i2c_speed;
	int board_i2cBus=board_i2c_bus[cur_i2c_bus];
	if (board_i2cBus<0) {
		// invalid
		//return -1;
		ret=-1;
		goto errOut;
	}
	i2cHandle = I2C_open(board_i2cBus, &i2cParams);
	if (i2cHandle == NULL) {
	    // Error opening I2C
	    //return -2;
	    ret=-2;
	    goto errOut;
	}
	uint8_t writeBuffer[4];
	if (alen>sizeof(writeBuffer)) {
		// too large addr
		//return -3;
		ret=-3;
		goto errOut;
	}
	int i;
	for (i=0; i<alen; i++) {
		writeBuffer[i]=addr>>(8*i);
	}
	I2C_Transaction i2cTransaction;
	i2cTransaction.slaveAddress = chip;
	i2cTransaction.writeBuf = writeBuffer;
	i2cTransaction.writeCount = alen;
	i2cTransaction.readBuf = buffer;
	i2cTransaction.readCount = len;
	bool status = I2C_transfer(i2cHandle, &i2cTransaction);
	if (status == false) {
	    // Unsuccessful I2C transfer
		//return -4;
		ret=-4;
		goto errOut;
	}
	I2C_close(i2cHandle);
	return 0;
errOut:
	I2C_close(i2cHandle);
	return ret;
}
int i2c_write(uint8_t chip, unsigned int addr, int alen, uint8_t *buffer, int len)
{
	int ret=0;
	I2C_Handle i2cHandle=NULL;
	I2C_Params i2cParams;
	I2C_Params_init(&i2cParams);
	i2cParams.transferMode = I2C_MODE_BLOCKING;
	//i2cParams.transferCallbackFxn = myCallbackFunction;
	i2cParams.bitRate = cur_i2c_speed;
	int board_i2cBus=board_i2c_bus[cur_i2c_bus];
	if (board_i2cBus<0) {
		// invalid
		//return -1;
		ret=-1;
		goto errOut;
	}
	i2cHandle = I2C_open(board_i2cBus, &i2cParams);
	if (i2cHandle == NULL) {
	    // Error opening I2C
	   // return -2;
		ret=-2;
		goto errOut;
	}
	static uint8_t writeBuffer[128];
	if (alen+len>sizeof(writeBuffer)) {
		// too large addr+data
		//return -3;
		ret=-3;
		goto errOut;
	}
	int i;
	for (i=0; i<alen; i++) {
		writeBuffer[i]=addr>>(8*i);
	}
	memmove(&writeBuffer[i],buffer,len);
	I2C_Transaction i2cTransaction;
	i2cTransaction.slaveAddress = chip;
	i2cTransaction.writeBuf = writeBuffer;
	i2cTransaction.writeCount = alen+len;
	i2cTransaction.readBuf = NULL;
	i2cTransaction.readCount = 0;
	bool status = I2C_transfer(i2cHandle, &i2cTransaction);
	if (status == false) {
	    // Unsuccessful I2C transfer
		//return -4;
		ret=-4;
		goto errOut;
	}
	I2C_close(i2cHandle);
	return 0;
errOut:
	I2C_close(i2cHandle);
	return ret;
}
#ifdef CONFIG_SYS_I2C
unsigned int i2c_set_bus_speed(unsigned int speed)
{
	if (speed<=board_i2c_speed[0]) {
		cur_i2c_speed=I2C_100kHz;
		return board_i2c_speed[0];
	}
	int i;
	for (i=1; i<board_i2c_speed_count; i++) {
		if (speed<board_i2c_speed[i]) {
			cur_i2c_speed=(I2C_BitRate)i;
			return board_i2c_speed[i-1];
		}
	}
	cur_i2c_speed=(I2C_BitRate)(board_i2c_speed_count-1);
	return board_i2c_speed[board_i2c_speed_count-1];
}
#else //-CONFIG_SYS_I2C
/*unsigned*/ int i2c_set_bus_speed(unsigned int speed)
{
	if (speed<=board_i2c_speed[0]) {
		cur_i2c_speed=I2C_100kHz;
		return 0; //board_i2c_speed[0];
	}
	int i;
	for (i=1; i<board_i2c_speed_count; i++) {
		if (speed<board_i2c_speed[i]) {
			cur_i2c_speed=(I2C_BitRate)i;
			return 0; //board_i2c_speed[i-1];
		}
	}
	cur_i2c_speed=(I2C_BitRate)(board_i2c_speed_count-1);
	return 0; //board_i2c_speed[board_i2c_speed_count-1];
}
#endif //-CONFIG_SYS_I2C
unsigned int i2c_get_bus_speed(void)
{
	return board_i2c_speed[cur_i2c_speed];
}
#endif //CONFIG_CMD_I2C
//============================



//============================
// SPI
//----------------------------
#ifdef CONFIG_CMD_SPI
//----------------------------
// avoid SPI_SLAVE conflict
# define SPI_SLAVE _SPI_SLAVE
#include <ti/drivers/SPI.h>
#undef SPI_SLAVE	// use _SPI_SLAVE as SPI_SLAVE
//----------------------------
#include "spi.h"	// u-boot header
//----------------------------
static const signed char board_spi_bus[]={
 // spi0 1           2             3
 -1, -1, MSP_EXP432E401Y_SPI2, MSP_EXP432E401Y_SPI3,
};
#define board_spi_bus_count	(sizeof(board_spi_bus)/sizeof(board_spi_bus[0]))
//static const int board_spi_bus_count=sizeof(board_spi_bus)/sizeof(board_spi_bus[0]);
static struct spi_slave uboot_spi_slave[board_spi_bus_count]={ 0, };
static unsigned char uboot_spi_slave_busy[board_spi_bus_count]={ 0, };
#ifndef MAX_SPI_BYTES	// see spi.c (u-boot cmd)
#   define MAX_SPI_BYTES 32	/* Maximum number of bytes we can handle */
#endif
//----------------------------
/**
 * Set up communications parameters for a SPI slave.
 *
 * This must be called once for each slave. Note that this function
 * usually doesn't touch any actual hardware, it only initializes the
 * contents of spi_slave so that the hardware can be easily
 * initialized later.
 *
 * @bus:	Bus ID of the slave chip.
 * @cs:		Chip select ID of the slave chip on the specified bus.
 * @max_hz:	Maximum SCK rate in Hz.
 * @mode:	Clock polarity, clock phase and other parameters.
 *
 * Returns: A spi_slave reference that can be used in subsequent SPI
 * calls, or NULL if one or more of the parameters are not supported.
 */
struct spi_slave *spi_setup_slave(unsigned int bus, unsigned int cs,
		unsigned int max_hz, unsigned int mode)
{
	if (bus<0 || board_spi_bus_count<=bus) {
		// no such bus
		return NULL;
	}
	if (board_spi_bus[bus]<0) {
		// no spi device
		return NULL;
	}
	if (uboot_spi_slave_busy[bus]) {
		// is busy
		return NULL;
	}
	struct spi_slave* slave= &uboot_spi_slave[bus];
	slave->bus=bus;
	slave->cs=cs;
	slave->mode=mode;
	//
	slave->max_hz=max_hz;
	//
	slave->wordlen=SPI_DEFAULT_WORDLEN;
	slave->max_read_size=MAX_SPI_BYTES;
	slave->max_write_size=MAX_SPI_BYTES;
	slave->memory_map=NULL;
	slave->flags=0; // note. SSIxfss=CS0# on SSIx bus
	//
	return slave;
}
/**
 * Free any memory associated with a SPI slave.
 *
 * @slave:	The SPI slave
 */
void spi_free_slave(struct spi_slave *slave)
{
	if (slave==NULL) {
		return;
	}
	int bus=slave->bus;
	if (bus<0 || board_spi_bus_count<=bus) {
		// no such bus
		return;
	}
	if (board_spi_bus[bus]<0) {
		// no spi device
		return;
	}
	uboot_spi_slave_busy[bus]=0;
}
/**
 * Claim the bus and prepare it for communication with a given slave.
 *
 * This must be called before doing any transfers with a SPI slave. It
 * will enable and initialize any SPI hardware as necessary, and make
 * sure that the SCK line is in the correct idle state. It is not
 * allowed to claim the same bus for several slaves without releasing
 * the bus in between.
 *
 * @slave:	The SPI slave
 *
 * Returns: 0 if the bus was claimed successfully, or a negative value
 * if it wasn't.
 */
int spi_claim_bus(struct spi_slave *slave)
{
	if (slave==NULL) { return -1; }
	int bus=slave->bus;
	if (bus<0 || board_spi_bus_count<=bus) {
		// no such bus
		return -1;
	}
	if (board_spi_bus[bus]<0) {
		// no spi device
		return -1;
	}
	// ok
	return 0;
}
/**
 * Release the SPI bus
 *
 * This must be called once for every call to spi_claim_bus() after
 * all transfers have finished. It may disable any SPI hardware as
 * appropriate.
 *
 * @slave:	The SPI slave
 */
void spi_release_bus(struct spi_slave *slave)
{
	return;
}

// setup SPI_Params
static int spi_get_SPI_Params(struct spi_slave *slave, SPI_Params* spiParams)
{
	if (slave==NULL) { return -1; }
	if (spiParams==NULL) { return -1; }
	int bus=slave->bus;
	if (bus<0 || board_spi_bus_count<=bus) {
		// no such bus
		return -1;
	}
	if (board_spi_bus[bus]<0) {
		// no spi device
		return -1;
	}

	//const int board_spiBus=board_spi_bus[slave->bus];

	//SPI_Params      spiParams;
	SPI_Params_init(spiParams);  // Initialize SPI parameters
	spiParams->transferMode = SPI_MODE_BLOCKING;
	spiParams->mode=SPI_MASTER;
	spiParams->bitRate=slave->max_hz;
	spiParams->dataSize=slave->wordlen;       // 8bit data size
	//
	spiParams->frameFormat=SPI_POL0_PHA0;
	int spi_mode=slave->mode;
	switch (spi_mode & SPI_MODE_3) {
	case SPI_MODE_0:
		//spiParams->frameFormat=SPI_POL0_PHA0;
		break;
	case SPI_MODE_1:	//(0|SPI_CPHA)
		spiParams->frameFormat=SPI_POL0_PHA1;
		break;
	case SPI_MODE_2:	//(SPI_CPOL|0)
		spiParams->frameFormat=SPI_POL1_PHA0;
		break;
	case SPI_MODE_3:	//(SPI_CPOL|SPI_CPHA)
		spiParams->frameFormat=SPI_POL1_PHA1;
		break;
	}
	// all other mode flags are ignored.
	//#define SPI_CS_HIGH	BIT(2)			/* CS active high */
	//#define SPI_LSB_FIRST	BIT(3)			/* per-word bits-on-wire */
	//#define SPI_3WIRE	BIT(4)			/* SI/SO signals shared */
	//#define SPI_LOOP	BIT(5)			/* loopback mode */
	//#define SPI_SLAVE	BIT(6)			/* slave mode */
	//#define SPI_PREAMBLE	BIT(7)			/* Skip preamble bytes */
	//#define SPI_TX_BYTE	BIT(8)			/* transmit with 1 wire byte */
	//#define SPI_TX_DUAL	BIT(9)			/* transmit with 2 wires */
	//#define SPI_TX_QUAD	BIT(10)			/* transmit with 4 wires */
	//#define SPI_RX_SLOW	BIT(11)			/* receive with 1 wire slow */
	//#define SPI_RX_DUAL	BIT(12)			/* receive with 2 wires */
	//#define SPI_RX_QUAD	BIT(13)			/* receive with 4 wires */
	//
	spiParams->custom=NULL;
	//
	return 0;
}

static SPI_Handle spiHandle_as_slave=NULL;
static int spi_xfer_as_slave_done=0;
// callback  of spi_xfer_as_slave()
static void transferCompleteFxn(SPI_Handle handle, SPI_Transaction *transaction)
{
    //sem_post(&slaveSem);
	spi_xfer_as_slave_done=1;
}
// close spi as slave
void spi_close_as_slave(void)
{
	if (spiHandle_as_slave) {
		if (spi_xfer_as_slave_done==0) {
			// cancel it
			SPI_transferCancel(spiHandle_as_slave);
		}
		if (spi_xfer_as_slave_done) {
			SPI_close(spiHandle_as_slave);
			spiHandle_as_slave=NULL;
		}
		spi_xfer_as_slave_done=0;
	}
}
// SPI transfer as SLAVE.  this return after setup receive. real receive done is by callback.
int  spi_xfer_as_slave(struct spi_slave *slave, unsigned int bitlen, const void *dout,
		void *din, unsigned long flags)
{
	spi_close_as_slave();

	static SPI_Params spiParams;
	if (spi_get_SPI_Params(slave,&spiParams)) {
		return -1;
	}
	spiParams.transferMode = SPI_MODE_CALLBACK;
	spiParams.mode= _SPI_SLAVE;	// SPI_SLAVE (avoiding conflict)
    spiParams.transferCallbackFxn = transferCompleteFxn;

	const int board_spiBus=board_spi_bus[slave->bus];

	spiHandle_as_slave = SPI_open(board_spiBus, &spiParams);
	if (spiHandle_as_slave == NULL) {
		return -2;  // SPI_open() failed
	}
	static SPI_Transaction spiTransaction={0,};
	spiTransaction.count = (bitlen+slave->wordlen-1)/slave->wordlen;
	spiTransaction.txBuf = dout;
	spiTransaction.rxBuf = din;
	bool transferOK = SPI_transfer(spiHandle_as_slave, &spiTransaction);
	if (!transferOK) {
	    // Error in SPI or transfer already in progress.
		SPI_close(spiHandle_as_slave);
		return -3;
	}
	// success
	// not SPI trans complete
	//SPI_close(spiHandle_as_slave);
	//
	return 0;
}

/**
 * SPI transfer
 *
 * This writes "bitlen" bits out the SPI MOSI port and simultaneously clocks
 * "bitlen" bits in the SPI MISO port.  That's just the way SPI works.
 *
 * The source of the outgoing bits is the "dout" parameter and the
 * destination of the input bits is the "din" parameter.  Note that "dout"
 * and "din" can point to the same memory location, in which case the
 * input data overwrites the output data (since both are buffered by
 * temporary variables, this is OK).
 *
 * spi_xfer() interface:
 * @slave:	The SPI slave which will be sending/receiving the data.
 * @bitlen:	How many bits to write and read.
 * @dout:	Pointer to a string of bits to send out.  The bits are
 *		held in a byte array and are sent MSB first.
 * @din:	Pointer to a string of bits that will be filled in.
 * @flags:	A bitwise combination of SPI_XFER_* flags.
 *
 * Returns: 0 on success, not 0 on failure
 */
int  spi_xfer(struct spi_slave *slave, unsigned int bitlen, const void *dout,
		void *din, unsigned long flags)
{
#if 0
	if (slave==NULL) { return -1; }
	int bus=slave->bus;
	if (bus<0 || board_spi_bus_count<=bus) {
		// no such bus
		return -1;
	}
	if (board_spi_bus[bus]<0) {
		// no spi device
		return -1;
	}

	const int board_spiBus=board_spi_bus[slave->bus];

	SPI_Params      spiParams;
	SPI_Params_init(&spiParams);  // Initialize SPI parameters
	spiParams.transferMode = SPI_MODE_BLOCKING;
	spiParams.mode=SPI_MASTER;
	spiParams.bitRate=slave->max_hz;
	spiParams.dataSize=slave->wordlen;       // 8bit data size
	//
	spiParams.frameFormat=SPI_POL0_PHA0;
	int spi_mode=slave->mode;
	switch (spi_mode & SPI_MODE_3) {
	case SPI_MODE_0:
		//spiParams.frameFormat=SPI_POL0_PHA0;
		break;
	case SPI_MODE_1:	//(0|SPI_CPHA)
		spiParams.frameFormat=SPI_POL0_PHA1;
		break;
	case SPI_MODE_2:	//(SPI_CPOL|0)
		spiParams.frameFormat=SPI_POL1_PHA0;
		break;
	case SPI_MODE_3:	//(SPI_CPOL|SPI_CPHA)
		spiParams.frameFormat=SPI_POL1_PHA1;
		break;
	}
	// all other mode flags are ignored.
	//#define SPI_CS_HIGH	BIT(2)			/* CS active high */
	//#define SPI_LSB_FIRST	BIT(3)			/* per-word bits-on-wire */
	//#define SPI_3WIRE	BIT(4)			/* SI/SO signals shared */
	//#define SPI_LOOP	BIT(5)			/* loopback mode */
	//#define SPI_SLAVE	BIT(6)			/* slave mode */
	//#define SPI_PREAMBLE	BIT(7)			/* Skip preamble bytes */
	//#define SPI_TX_BYTE	BIT(8)			/* transmit with 1 wire byte */
	//#define SPI_TX_DUAL	BIT(9)			/* transmit with 2 wires */
	//#define SPI_TX_QUAD	BIT(10)			/* transmit with 4 wires */
	//#define SPI_RX_SLOW	BIT(11)			/* receive with 1 wire slow */
	//#define SPI_RX_DUAL	BIT(12)			/* receive with 2 wires */
	//#define SPI_RX_QUAD	BIT(13)			/* receive with 4 wires */
	//
	spiParams.custom=NULL;
#endif //0
	SPI_Params spiParams;
	if (spi_get_SPI_Params(slave,&spiParams)) {
		return -1;
	}
	const int board_spiBus=board_spi_bus[slave->bus];

	SPI_Handle spiHandle = SPI_open(board_spiBus, &spiParams);
	if (spiHandle == NULL) {
		return -2;  // SPI_open() failed
	}
	/*static*/ SPI_Transaction spiTransaction={0,};
	spiTransaction.count = (bitlen+slave->wordlen-1)/slave->wordlen;
	spiTransaction.txBuf = dout;
	spiTransaction.rxBuf = din;
	bool transferOK = SPI_transfer(spiHandle, &spiTransaction);
	if (!transferOK) {
	    // Error in SPI or transfer already in progress.
		SPI_close(spiHandle);
		return -3;
	}
	// success
	SPI_close(spiHandle);
	//
	return 0;
}
#endif //CONFIG_CMD_SPI
//============================


//============================
// ADC
//----------------------------
#ifdef CONFIG_CMD_ADC
#include <ti/devices/msp432e4/driverlib/adc.h>
#include <ti/drivers/adc/ADCMSP432E4.h>
#include <ti/drivers/adcbuf/ADCBufMSP432E4.h>
#include <ti/drivers/ADC.h>
#include <ti/drivers/ADCBuf.h>
#include <semaphore.h>
#include "MSP_EXP432E401Y.h"

int pinBit2pin(const int pinBit) {
	switch (pinBit) {
	case 1: return 0;
	case 2: return 1;
	case 4: return 2;
	case 8: return 3;
	case 16: return 4;
	case 32: return 5;
	case 64: return 6;
	case 128: return 7;
	}
	return -1;
}
int portAddr2port(const int portAddr) {
	const int x=(portAddr>>12)&0xFF;	// 0x59 -> PB
	// PIx, POx is missing
	int port=x-0x58;
	return port;
}
char portAddr2portChar(const int portAddr) {
	int port=portAddr2port(portAddr);
	const char str[]="ABCDEFGHJKLMNPQ";
	if (port>=sizeof(str)-1) { return ' '; }
	return str[port];
}
static char adc_pinstr_buf[16];
const char* adc_pinstr(const uint32_t pinconf) {
	int chan=(pinconf>>16)&0x1F;
	int portAddr=((pinconf<<4)&0xFF000)|0x40000000;
	int pinBit=pinconf&0xFF;
	// "A10(PB4)"
	sprintf(adc_pinstr_buf,"A%d(P%c%d)",chan,portAddr2portChar(portAddr),pinBit2pin(pinBit));
	return adc_pinstr_buf;
};
const char* adc_chanstr(const uint32_t pinconf) {
	int chan=(pinconf>>16)&0x1F;
	//int portAddr=((pinconf<<4)&0xFF000)|0x40000000;
	//int pinBit=pinconf&0xFF;
	// PB4 A10
	sprintf(adc_pinstr_buf,"A%d",chan);
	return adc_pinstr_buf;
}
//----------------------------
// adc chan information
//----------------------------
static int disp_adc_chan_info(const int adcbuf_mod, const int adcbuf_ch)
{
	if (adcbuf_ch<0 || MSP_EXP432E401Y_ADCBUF0CHANNELCOUNT<=adcbuf_ch) {
		return -1;
	}
	// extend information
	extern ADCBufMSP432E4_Channels adcBuf0MSP432E4Channels[MSP_EXP432E401Y_ADCBUF0CHANNELCOUNT];
	ADCBufMSP432E4_Channels* info= &adcBuf0MSP432E4Channels[adcbuf_ch];
	printf("ch%d : ", adcbuf_ch);
    /*! ADC reference voltage in microVolts */
	printf("ref=%duV ", info->refVoltage);

	if (info->adcInputMode==ADCBufMSP432E4_SINGLE_ENDED) {
		// single-end
		if (info->adcInternalSource==ADCBufMSP432E4_INTERNAL_SOURCE_MODE_OFF) {
			printf("Pin=%s ", adc_pinstr(info->adcPin));
		}
		else {
			printf("%s_InternalSensor ",adc_chanstr(info->adcPin));
		}
		if (info->adcDifferentialPin!=ADCBufMSP432E4_PIN_NONE) {
			// illegal setting. diff pin is used!?
			printf("\nILLEAGAL SETTING. SingleEnd mode selected. but adcDifferentialPin IS assigned.\n");
		}
	}
	else if (info->adcInputMode==ADCBufMSP432E4_DIFFERENTIAL) {
		// differential
		printf("Pin=%s ", adc_pinstr(info->adcPin));
		if (info->adcDifferentialPin==ADCBufMSP432E4_PIN_NONE) {
			// illegal setting. diff pin is NONE!?
			printf("\nILLEAGAL SETTING. Differential mode selected. but adcDifferentialPin is not assigned.\n");
		}
		else {
			printf(" Diff.with=%s ", adc_pinstr(info->adcDifferentialPin));
		}
		if (info->adcInternalSource!=ADCBufMSP432E4_INTERNAL_SOURCE_MODE_OFF) {
			// illegal setting. internal must be SingleEnd
			printf("\nILLEGAL SETTING. Internal source cannot use at Differential mode.\n");
		}
	}
	else {
		// unknown
		printf("\nILLEGAL SETTING. unknown InputMode\n");
	}
	printf("\n");
	return 0;
}
int ti_rtos_adc_info(const int adcbuf_mod, const int adcbuf_ch)
{
	if (adcbuf_mod<0||MSP_EXP432E401Y_ADCBUFCOUNT<=adcbuf_mod) {
		// no such ADC module
		printf("No such ADC Device\n");
		return -3;
	}
	// basic information (depend to ADC module)
	// data mask
	const int data_mask=0xFFF;	// MSP432E401Y is 12bit ADC x2
	printf("data mask: %x\n", data_mask);
	// vdd (uV)
	const int vdd=3300*1000;	// Vdd=3.3V (not refVoltage)
	printf("vdd: %duV\n", vdd);
	// vss (uV)
	const int vss=0;	// Vss=0V
	printf("vss: %duV\n", vss);

	// extend information (depend to channel)
	if (adcbuf_ch<0) {
		// disp all
		int ch;
		for (ch=0; ch<MSP_EXP432E401Y_ADCBUF0CHANNELCOUNT; ch++) {
			disp_adc_chan_info(adcbuf_mod,ch);
		}
		return 0;
	}
	else {
		// disp chan only
		if (adcbuf_ch<0 || MSP_EXP432E401Y_ADCBUF0CHANNELCOUNT<=adcbuf_ch) {
			// no such chan
			return -1;
		}
		return disp_adc_chan_info(adcbuf_mod,adcbuf_ch);
	}
}
//----------------------------
// single shot (irq handler)
//----------------------------
#define ADCBufBUFFERSIZE 20
static uint16_t adcbuf_samplebuf1[ADCBufBUFFERSIZE];
//static uint16_t adcbuf_samplebuf2[ADCBufBUFFERSIZE];
static uint32_t adcbuf_uV[ADCBufBUFFERSIZE];
static uint16_t adcbuf_val[ADCBufBUFFERSIZE];
static sem_t adcbuf_sem;
static void adc_single_callback(
		ADCBuf_Handle handle, ADCBuf_Conversion* conversion,
		void* completedADCBuffer, uint32_t completedChannel )
{
	// sample value
	memmove(adcbuf_val,completedADCBuffer,sizeof(adcbuf_val));
	// convert to uV
	ADCBuf_adjustRawValues(handle,completedADCBuffer, ADCBufBUFFERSIZE,completedChannel);
	ADCBuf_convertAdjustedToMicroVolts(handle,completedChannel,completedADCBuffer,adcbuf_uV,ADCBufBUFFERSIZE);
	// tell adc done
	sem_post(&adcbuf_sem);

}
//----------------------------
// single shot main
//----------------------------
int ti_rtos_adc_single(const int adcbuf_mod, const int adcbuf_ch, unsigned long* uVp)
{
	// must be called after ADCBuf_init();

	// check arg params
	if (adcbuf_mod<0||MSP_EXP432E401Y_ADCBUFCOUNT<=adcbuf_mod) {
		// no such ADC module
		return -3;
	}
	if (adcbuf_ch<0||MSP_EXP432E401Y_ADCBUF0CHANNELCOUNT<=adcbuf_ch) {
		// no such channel
		return -3;
	}

	//
	static ADCBuf_Handle s_psADCBufHandle[MSP_EXP432E401Y_ADCBUFCOUNT]={ 0, };
	static unsigned char semInited=0;
	if (semInited==0) {
		if (sem_init(&adcbuf_sem,0,0)) {
			// semafore error
			return -1;
		}
		semInited=1;
	}

	if (s_psADCBufHandle[adcbuf_mod]==NULL) {
		// ADCBuf open (only once; never close for ADCBuf driver's bug)
		ADCBuf_Params adcBufParam;
		ADCBuf_Params_init(&adcBufParam);
		adcBufParam.callbackFxn=adc_single_callback;
		adcBufParam.recurrenceMode=ADCBuf_RECURRENCE_MODE_ONE_SHOT;
		adcBufParam.returnMode=ADCBuf_RETURN_MODE_CALLBACK;
		adcBufParam.samplingFrequency=200;
		// open ADC
		s_psADCBufHandle[adcbuf_mod]=ADCBuf_open(adcbuf_mod,&adcBufParam);
		if (s_psADCBufHandle[adcbuf_mod]==NULL) {
			// fail
			return -2;
		}
	}
	//
	ADCBuf_Handle g_psADCBufHandle=s_psADCBufHandle[adcbuf_mod];
	// conversion config
	ADCBuf_Conversion convParam;
	convParam.arg=NULL;
	convParam.adcChannel=adcbuf_ch;
	convParam.sampleBuffer=adcbuf_samplebuf1;
	convParam.sampleBufferTwo=adcbuf_samplebuf1;
	convParam.samplesRequestedCount=ADCBufBUFFERSIZE;
	// ADCBuf start
	if (ADCBuf_convert(g_psADCBufHandle,&convParam,1)!=ADCBuf_STATUS_SUCCESS) {
		// fail
		ADCBuf_close(g_psADCBufHandle);
		return -3;
	}
	// wait adc done
	sem_wait(&adcbuf_sem);
	// DONT! never close! avoid for ADCBuf driver's bug.
	// ADCBuf close
	//ADCBuf_close(g_psADCBufHandle);

	// averaging
	unsigned long sum=0;
	int i;
	for (i=0; i<ADCBufBUFFERSIZE; i++) {
		sum+=adcbuf_val[i];
	}
	sum/=ADCBufBUFFERSIZE;
	if (uVp) {
		unsigned long long sum_uV=0;
		for (i=0; i<ADCBufBUFFERSIZE; i++) {
			sum_uV+=adcbuf_uV[i];
		}
		sum_uV/=ADCBufBUFFERSIZE;
		*uVp=sum_uV;
	}
	//
	return sum;
}
#endif //CONFIG_CMD_ADC
//============================


//============================
// USB (host)
//----------------------------
// lower level I/O for u-boot
//----------------------------
#ifdef CONFIG_CMD_USB
#include <ti/usblib/msp432e4/usb-ids.h>
#include <ti/usblib/msp432e4/usblib.h>
#include <ti/usblib/msp432e4/usbhid.h>
#include <ti/usblib/msp432e4/host/usbhost.h>
#include <ti/usblib/msp432e4/host/usbhostpriv.h>
#include <ti/usblib/msp432e4/host/usbhmsc.h>
//----------------------------
#include "usb.h"	// u-boot header
//#include "mapmem.h"	// u-boot header
#include "memalign.h"	// u-boot header
//----------------------------
static struct usb_device usbDev={
	 .devnum=1,	// the device numbering starts at 1 on each bus.
};
static int asynch_allowed;
//----------------------------
// cannot move to USBMSCH.h
// it cause error in ITaskSupport.h. 'struct usb_device' is too bad.
extern int USBMSCH_userWaitOnLine(struct usb_device* usbDev);
extern tUSBHostDevice* USBMSCH_get_USBHostDevice(struct usb_device* ti_usbDev);
//----------------------------
/* initialize the USB Controller */
int usb_init(void)
{
	asynch_allowed = 1;
	USBMSCH_init(Board_USBHOST);	// HOST; if Deivce, use Board_USBDEVICE, or Board_USBULPI (but to be use USBMSCD_init())
 	int sts=USBMSCH_userWaitOnLine(&usbDev);
 	if (sts) {
 		// error
 		USBMSCH_finish();
 	}
 	return sts;
}
/* stop the USB Controller */
int usb_stop(void)
{
	asynch_allowed = 1;
	//usb_hub_reset();
	return USBMSCH_finish();
}
struct usb_device *usb_get_dev_index(int index)
{
	if (index==0) {
		return &usbDev;
	}
	// no such USB device
	return NULL;
}
int submit_control_msg(struct usb_device *dev, unsigned long pipe, void *buffer,
			int transfer_len, struct devrequest *setup)
{
	tUSBHostDevice* psDevice=USBMSCH_get_USBHostDevice(dev);
	if (psDevice==NULL) { return -EINVAL; }

	tUSBRequest* psSetupPacket=(tUSBRequest*)setup;
	uint32_t maxPacketSize=psDevice->sDeviceDescriptor.bMaxPacketSize0;
	uint32_t written=USBHCDControlTransfer(0,psSetupPacket,psDevice,buffer,transfer_len,maxPacketSize);
	if (written==transfer_len) {
		// success
		dev->status=0;
		dev->act_len=transfer_len;
		return written;
	}
	// fail
	dev->status=USB_ST_NOT_PROC;
	dev->act_len=0;
	return -EIO;
}
# ifdef CONFIG_USB_STORAGE
int submit_bulk_msg(struct usb_device *dev, unsigned long pipe,
			void *buffer, int transfer_len)
{
	dev->act_len=0;
	dev->status=USB_ST_NOT_PROC;	// as error

	if (! usb_pipebulk(pipe)) {
		// not bulk pipe
		return -EINVAL;
	}
	if (g_psMSCInstance==NULL) {
		// not usb setupped
		return -EINVAL;
	}
	if (usb_pipeout(pipe)) {
		// out pipe
		//g_psMSCInstance->pfnCallback=usb_pipeout_done;
		debug("bulk out pipe=%x\n",g_psMSCInstance->ui32BulkInPipe);
		dev->act_len=USBHCDPipeWrite(g_psMSCInstance->ui32BulkOutPipe,buffer,transfer_len);
		//g_psMSCInstance->pfnCallback=NULL;
	}
	else {
		// in pipe
		//g_psMSCInstance->pfnCallback=usb_pipein_done;
		debug("bulk in pipe=%x\n",g_psMSCInstance->ui32BulkInPipe);
		dev->act_len=USBHCDPipeRead(g_psMSCInstance->ui32BulkInPipe,buffer,transfer_len);
		//g_psMSCInstance->pfnCallback=NULL;
	}
	if (dev->act_len==transfer_len) {
		// success
		dev->status=0;
		return 0;
	}
	// some error
	return -EIO;
}
/*
 * submits an Interrupt Message
 */
int submit_int_msg(struct usb_device *dev, unsigned long pipe,
			void *buffer, int transfer_len, int interval)
{
	// now not supported
	return -EINVAL;
}
# endif //CONFIG_USB_STORAGE
//============================
// u-boot side wrap functions
//----------------------------
/********************************************************************
 * usb_string:
 * Get string index and translate it to ascii.
 * returns string length (> 0) or error (< 0)
 */
int usb_string(struct usb_device *dev, int index, char *buf, size_t size)
{
	// now not supported
	return 0;
}
/*
 * submits a control message and waits for comletion (at least timeout * 1ms)
 * If timeout is 0, we don't wait for completion (used as example to set and
 * clear keyboards LEDs). For data transfers, (storage transfers) we don't
 * allow control messages with 0 timeout, by previousely resetting the flag
 * asynch_allowed (usb_disable_asynch(1)).
 * returns the transferred length if OK or -1 if error. The transferred length
 * and the current status are stored in the dev->act_len and dev->status.
 */
int usb_control_msg(struct usb_device *dev, unsigned int pipe,
			unsigned char request, unsigned char requesttype,
			unsigned short value, unsigned short index,
			void *data, unsigned short size, int timeout)
{
	ALLOC_CACHE_ALIGN_BUFFER(struct devrequest, setup_packet, 1);
	int err;

	if ((timeout == 0) && (!asynch_allowed)) {
		/* request for a asynch control pipe is not allowed */
		return -EINVAL;
	}

	/* set setup command */
	setup_packet->requesttype = requesttype;
	setup_packet->request = request;
	setup_packet->value = cpu_to_le16(value);
	setup_packet->index = cpu_to_le16(index);
	setup_packet->length = cpu_to_le16(size);
	debug("usb_control_msg: request: 0x%X, requesttype: 0x%X, " \
	      "value 0x%X index 0x%X length 0x%X\n",
	      request, requesttype, value, index, size);
	dev->status = USB_ST_NOT_PROC; /*not yet processed */

	err = submit_control_msg(dev, pipe, data, size, setup_packet);
	if (err < 0)
		return err;
	if (timeout == 0)
		return (int)size;

	/*
	 * Wait for status to update until timeout expires, USB driver
	 * interrupt handler may set the status when the USB operation has
	 * been completed.
	 */
	while (timeout--) {
		if (!((volatile unsigned long)dev->status & USB_ST_NOT_PROC))
			break;
		mdelay(1);
	}
	if (dev->status)
		return -1;

	return dev->act_len;
}
/*-------------------------------------------------------------------
 * submits bulk message, and waits for completion. returns 0 if Ok or
 * negative if Error.
 * synchronous behavior
 */
# ifdef CONFIG_USB_STORAGE
int usb_bulk_msg(struct usb_device *dev, unsigned int pipe,
			void *data, int len, int *actual_length, int timeout)
{
	if (len < 0)
		return -EINVAL;
	dev->status = USB_ST_NOT_PROC; /*not yet processed */
	if (submit_bulk_msg(dev, pipe, data, len) < 0)
		return -EIO;
	while (timeout--) {
		if (!((volatile unsigned long)dev->status & USB_ST_NOT_PROC))
			break;
		mdelay(1);
	}
	*actual_length = dev->act_len;
	if (dev->status == 0)
		return 0;
	else
		return -EIO;
}

/*-------------------------------------------------------------------
 * Max Packet stuff
 */

/*
 * returns the max packet size, depending on the pipe direction and
 * the configurations values
 */
int usb_maxpacket(struct usb_device *dev, unsigned long pipe)
{
	/* direction is out -> use emaxpacket out */
	if ((pipe & USB_DIR_IN) == 0)
		return dev->epmaxpacketout[((pipe>>15) & 0xf)];
	else
		return dev->epmaxpacketin[((pipe>>15) & 0xf)];
}
/***********************************************************************
 * Clears an endpoint
 * endp: endpoint number in bits 0-3;
 * direction flag in bit 7 (1 = IN, 0 = OUT)
 */
int usb_clear_halt(struct usb_device *dev, int pipe)
{
	int result;
	int endp = usb_pipeendpoint(pipe)|(usb_pipein(pipe)<<7);

	result = usb_control_msg(dev, usb_sndctrlpipe(dev, 0),
				 USB_REQ_CLEAR_FEATURE, USB_RECIP_ENDPOINT, 0,
				 endp, NULL, 0, USB_CNTL_TIMEOUT * 3);

	/* don't clear if failed */
	if (result < 0)
		return result;

	/*
	 * NOTE: we do not get status and verify reset was successful
	 * as some devices are reported to lock up upon this check..
	 */

	usb_endpoint_running(dev, usb_pipeendpoint(pipe), usb_pipeout(pipe));

	/* toggle is reset on clear */
	usb_settoggle(dev, usb_pipeendpoint(pipe), usb_pipeout(pipe), 0);
	return 0;
}
/********************************************************************
 * set interface number to interface
 */
int usb_set_interface(struct usb_device *dev, int interface, int alternate)
{
	struct usb_interface *if_face = NULL;
	int ret, i;

	for (i = 0; i < dev->config.desc.bNumInterfaces; i++) {
		if (dev->config.if_desc[i].desc.bInterfaceNumber == interface) {
			if_face = &dev->config.if_desc[i];
			break;
		}
	}
	if (!if_face) {
		printf("selecting invalid interface %d", interface);
		return -EINVAL;
	}
	/*
	 * We should return now for devices with only one alternate setting.
	 * According to 9.4.10 of the Universal Serial Bus Specification
	 * Revision 2.0 such devices can return with a STALL. This results in
	 * some USB sticks timeouting during initialization and then being
	 * unusable in U-Boot.
	 */
	if (if_face->num_altsetting == 1)
		return 0;

	ret = usb_control_msg(dev, usb_sndctrlpipe(dev, 0),
				USB_REQ_SET_INTERFACE, USB_RECIP_INTERFACE,
				alternate, interface, NULL, 0,
				USB_CNTL_TIMEOUT * 5);
	if (ret < 0)
		return ret;

	return 0;
}

/*
 * disables the asynch behaviour of the control message. This is used for data
 * transfers that uses the exclusiv access to the control and bulk messages.
 * Returns the old value so it can be restored later.
 */
int usb_disable_asynch(int disable)
{
	int old_value = asynch_allowed;

	asynch_allowed = !disable;
	return old_value;
}
# endif //CONFIG_USB_STORAGE
#endif //CONFIG_CMD_USB
//============================



//============================
// udevice for TI RTOS
//----------------------------
//----------------------------
int _uclass_find_device_by_id(int index, struct udevice **devp)
{
	if (devp==NULL) { return -1; }

	// find by index
	for (;;) {
		if ((*devp)->id<0) {
			// no device
			*devp=NULL;
			return -ENODEV;
		}
		if ((*devp)->id==index) {
			// match
			return 0;
		}
		// next
		(*devp)++;
	}
	// never
	return -ENODEV;
}
//----------------------------
int uclass_get_device(enum uclass_id id, int index, struct udevice **devp)
{
	if (devp==NULL) { return -1; }

	int sts=uclass_first_device_err(id,devp);
	if (sts) {
		return sts;
	}
	return _uclass_find_device_by_id(index,devp);
}
//----------------------------
int uclass_get_device_by_name(enum uclass_id id, const char *name,
			      struct udevice **devp)
{
	if (devp==NULL) { return -1; }

	int sts=uclass_first_device_err(id,devp);
	if (sts) {
		return sts;
	}
	// find by name
	for (;;) {
		if ((*devp)->id<0) {
			// no device
			*devp=NULL;
			return -ENODEV;
		}
		if (strcmp((*devp)->name,name)==0) {
			// match
			return 0;
		}
		// next
		(*devp)++;
	}
	// never
	return -ENODEV;
}
//----------------------------
#ifdef CONFIG_CMD_ADC
extern struct udevice adc_udev[];
#endif //CONFIG_CMD_ADC
//----------------------------
int uclass_first_device_err(enum uclass_id id, struct udevice **devp)
{
	if (devp==NULL) { return -1; }

	*devp=NULL;

	switch (id) {
#ifdef CONFIG_CMD_ADC
	case UCLASS_ADC:
		if (adc_udev[0].id>=0) {
			*devp = &adc_udev[0];
			return 0;
		}
		break;
#endif //CONFIG_CMD_ADC
	}
	// not supported
	return -ENODEV;
}
//----------------------------
int uclass_next_device(struct udevice **devp)
{
	if (devp==NULL || *devp==NULL) { return -1; }

	(*devp)++;
	if ((*devp)->id<0) {
		// no more
		*devp=NULL;
		return 0;
	}
	return 0;
}
//============================

#ifdef CONFIG_NET
static struct eth_device ethDev={
	 .index=0,	// eth starts 0, USB ->the device numbering starts at 1 on each bus.
	 .name="tiva",
};
/* get dev @ index */
struct eth_device *eth_get_dev_by_index(int index)
{
	if (index==0) {
		return &ethDev;
	}
	// no such eth device
	return NULL;
}
/* get name of current device */
const char *eth_get_name(void)
{
		return ethDev.name;
}
extern char g_pcMACAddress[MAC_ADDRESS_LENGTH + 1];
//extern uint32_t g_ui32IPAddr;
// get eth env "ethaddr","ethXaddr","ipaddr"
const char* get_eth_info(const char* attr)
{
	if (strcmp(attr,"ethaddr")==0 || strcmp(attr,"eth0addr")==0) {
		// return MAC addr of eth0
		/* Get MAC address.  If unsuccessful, exit. */
		return g_pcMACAddress;
	}
	if (strcmp(attr,"ipaddr")==0) {
		// return IP addr of eth0
		static char ipaddr_str[(3+1)*4]={0};
		sprintf(ipaddr_str,"%u.%u.%u.%u",g_ui32IPAddr&0xFF,(g_ui32IPAddr>>8)&0xFF,(g_ui32IPAddr>>16)&0xFF,(g_ui32IPAddr>>24)&0xFF);
		return ipaddr_str;
	}
	return "";
}
//============================
#include "NDK/netmain.h"
#include <ti/net/bsd/netdb.h>
#ifdef h_addr
# undef h_addr
#endif //h_addr
//#include "NDK/_stack.h"
//#include "NDK/console.h"
//#include <malloc.h>
//============================
// shit NDK. gethostbyname return always 0.
// so, i wrote subset(not-compatible) function. IPv4 only.
//============================
#define DNSGetHostByXXXX_BUFSIZE	512
static union {
	char buf[DNSGetHostByXXXX_BUFSIZE];
	HOSTENT ndk_hostent;
} ti_rtos_DNSGHB_buf;
static struct hostent ti_rtos_bsd_hostent;;
static unsigned long* ti_rtos_h_addr_list_ary[NDK_DNS_MAXIPADDR];
struct hostent* ti_rtos_gethostbyname(const char* name)
{
	// NDK

    /* We can treat buffer as a HOSTENT structure after */
    /* DNSGetHostByXxx calls */
	// working aria must be sizeof(HOSTENT)+ip,hostnameArea
    HOSTENT *phe = &ti_rtos_DNSGHB_buf.ndk_hostent;
    int retcode = DNSGetHostByName( (char*)name, &ti_rtos_DNSGHB_buf, sizeof(ti_rtos_DNSGHB_buf) );
    if( retcode ) {
    	// dns fail
    	return NULL;
    }
	// BSD
    ti_rtos_bsd_hostent.h_name=phe->h_name;
    ti_rtos_bsd_hostent.h_aliases=NULL;
    ti_rtos_bsd_hostent.h_addrtype=phe->h_addrtype;
    ti_rtos_bsd_hostent.h_length=phe->h_length;
    ti_rtos_bsd_hostent.h_addr_list=(char**)ti_rtos_h_addr_list_ary;
	int i;
	for (i=0; i<NDK_DNS_MAXIPADDR; i++) {
		if (i<phe->h_addrcnt) {
			ti_rtos_h_addr_list_ary[i]= (unsigned long*)&phe->h_addr[i]; //(unsigned long*)&ti_rtos_bsd_hostent.h_addr[i];
		}
		else {
			ti_rtos_h_addr_list_ary[i]=NULL;
		}
	}
	//ipaddr=0xC0A8015E;
	return &ti_rtos_bsd_hostent;
}
struct hostent* ti_rtos_gethostbyaddr(const void* ipaddrp, int/*socklen_t*/ len, int type)
{
	uint32_t ipaddr= *(uint32_t*)ipaddrp;
	// NDK
	static HOSTENT ndk_hostent;
    /* We can treat buffer as a HOSTENT structure after */
    /* DNSGetHostByXxx calls */
    HOSTENT *phe = &ndk_hostent;
    int retcode = DNSGetHostByAddr( ipaddr, &ndk_hostent, sizeof(ndk_hostent) );
    if( retcode ) {
    	// dns fail
    	return NULL;
    }
	// BSD
    ti_rtos_bsd_hostent.h_name=phe->h_name;
    ti_rtos_bsd_hostent.h_aliases=NULL;
    ti_rtos_bsd_hostent.h_addrtype=phe->h_addrtype;
    ti_rtos_bsd_hostent.h_length=phe->h_length;
    ti_rtos_bsd_hostent.h_addr_list=(char**)ti_rtos_h_addr_list_ary;
	int i;
	for (i=0; i<NDK_DNS_MAXIPADDR; i++) {
		if (i<phe->h_addrcnt) {
			ti_rtos_h_addr_list_ary[i]= (unsigned long*)&phe->h_addr[i];
		}
		else {
			ti_rtos_h_addr_list_ary[i]=NULL;
		}
	}
	//ipaddr=0xC0A8015E;
	return &ti_rtos_bsd_hostent;
}
#endif //CONFIG_NET
//============================

//============================
// cited from TI SDK
//============================
/*
 * Copyright (c) 2012-2018, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * */
/*
 * ======== console.c ========
 */
/*-------------------------------------------------------------- */
/* ConPrintIPN */
/* Quick routine to print out an uint32_t addr */
/*-------------------------------------------------------------- */
#include <NDK/socketndk.h>
void ti_rtos_printIPN( unsigned long /*uint32_t*/ IPAddr )
{
    IPAddr = NDK_htonl( IPAddr );
    ConPrintf( "%d.%d.%d.%d",
               (unsigned char)((IPAddr>>24)&0xFF), (unsigned char)((IPAddr>>16)&0xFF),
               (unsigned char)((IPAddr>>8)&0xFF), (unsigned char)(IPAddr&0xFF) );
}
//============================

//============================
// tftpget command (TI_RTOS original)
//============================
#ifdef CONFIG_CMD_TFTPGET
int do_tftpget(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	if (argc == 2) {
		return CMD_RET_USAGE;
	}

	// ip addr
	uint32_t ipaddr=0;
	struct in_addr tftp_server = string_to_ip(argv[1]);
	if (tftp_server.s_addr == 0) {
		printf("Bad TFTP server IP address\n");
		return CMD_RET_FAILURE;
	}
	//ipaddr=NDK_htonl(tftp_server.s_addr);
	ipaddr=tftp_server.s_addr;
	// filename
	char* File=argv[2];
	// address
	int addr=load_addr;
	if (argc>=4) {
		addr=simple_strtol(argv[3], NULL, 16);
	}
	// max size
#define MAX_TFTPGET_FILE_SIZE	(128*1024)
	int maxSize=MAX_TFTPGET_FILE_SIZE;

	int doTFTPget( uint32_t IPAddr, char *File , char* buffer, int maxSize);
	int sts=doTFTPget( ipaddr, File, (char*)addr, maxSize);
	if (sts) {
		return CMD_RET_FAILURE;
	}
	return CMD_RET_SUCCESS;
}
U_BOOT_CMD(
	tftpget,	4,	2,	do_tftpget,
	"retrieve file from TFTP server",
	"tftpServer file [address]"
);
#endif //CONFIG_CMD_TFTPGET
//============================


