// ti_rtos_ubootcmd_task.c


#include <stdbool.h>
#include <stdlib.h>
#include <pthread.h>

#include "Board_ubootcmd.h"
#include "board_funcs.h"
#include "cloud_task.h"		// for MAC_ADDRESS_LENGTH

#include "config.h"
#include "ti_rtos_ubootcmd_task.h"
#include "uart6.h"

// u-boot headers
#include "command.h"
#include "common.h"



// 2048 is too small. printf occurs hangup.
#define TI_RTOS_UBOOTCMD_TASKSTACKSIZE (2048*2)
// opening message
#define BANNER	"\r\n\r\nMSP-EXP432E401Y LaunchPad monitor\r\n"

extern char g_pcMACAddress[MAC_ADDRESS_LENGTH + 1];

//============================================================================
// ti_rtos_ubootcmd_task
//----------------------------------------------------------------------------
void* ti_rtos_ubootcmd_task(void *arg0)
{
    // initialize command table list
	extern void init_cmd_tbl_list(void);
	init_cmd_tbl_list();
#ifdef CONFIG_HAVE_BLOCK_DEVICE
	extern void init_blk_driver_list(void);
	init_blk_driver_list();
#endif //CONFIG_HAVE_BLOCK_DEVICE
#ifdef CONFIG_PARTITIONS
	extern void init_part_driver_list(void);
	init_part_driver_list();
#endif //CONFIG_PARTITIONS
#ifdef CONFIG_SYS_MSP432_FLASH	//CONFIG_CMD_FLASH
	flash_init();
#endif //CONFIG_SYS_MSP432_FLASH	//CONFIG_CMD_FLASH

	uboot_init();

#ifdef USE_UART6
	uart6_init(115200);
	puts_uart6_tx_buf(BANNER);
#endif //-USE_UART6

    // Get MAC address.  If unsuccessful, exit.
    if(GetMacAddress(g_pcMACAddress, sizeof(g_pcMACAddress)) == false)
    {
        // Return error.
#ifdef USE_UART6
        puts_uart6_tx_buf("Failed to get MAC address.  Looping forever.\r\n");
#endif //-USE_UART6
        while(1);
    }
	// moniter comman main loop
    main_loop();

    return NULL;
}
//============================================================================

//============================================================================
// init for ti_rtos_ubootcmd_task
//----------------------------------------------------------------------------
int32_t ti_rtos_ubootcmd_taskInit(void)
{
	pthread_t           thread;
	pthread_attr_t      attrs;
	struct sched_param  priParam;
	int                 retc;
	int                 detachState;

	/* Set priority and stack size attributes */
	pthread_attr_init(&attrs);
	priParam.sched_priority = 1;	// msut be 1 or lower?

	detachState = PTHREAD_CREATE_DETACHED;
	retc = pthread_attr_setdetachstate(&attrs, detachState);
	if (retc != 0) {
		/* pthread_attr_setdetachstate() failed */
		return (-1);
	}

	pthread_attr_setschedparam(&attrs, &priParam);

	retc |= pthread_attr_setstacksize(&attrs, TI_RTOS_UBOOTCMD_TASKSTACKSIZE);
	if (retc != 0) {
		/* pthread_attr_setstacksize() failed */
		return (-1);
	}

	retc = pthread_create(&thread, &attrs, ti_rtos_ubootcmd_task, NULL);
	if (retc != 0) {
		/* pthread_create() failed */
		return (-1);
	}

	return 0;
}
//============================================================================

//============================================================================
// init for ti_rtos_ubootcmd_task
#ifdef CONFIG_CMD_ADC
 void ConfigureADC(void);
#endif //CONFIG_CMD_ADC
#ifdef CONFIG_CMD_I2C
 void ConfigureI2C(void);
#endif //CONFIG_CMD_I2C
#ifdef CONFIG_CMD_SPI
 void ConfigureSPI(void);
#endif //CONFIG_CMD_I2C
#ifdef CONFIG_CMD_USB
 void ConfigureUSB(void);
#endif //CONFIG_CMD_USB
//----------------------------------------------------------------------------
void ti_rtos_ubootcmd_configure(void)
{
#ifdef CONFIG_CMD_ADC
    ConfigureADC();
#endif //CONFIG_CMD_ADC
#ifdef CONFIG_CMD_I2C
	ConfigureI2C();
#endif //CONFIG_CMD_I2C
#ifdef CONFIG_CMD_SPI
	ConfigureSPI();
#endif //CONFIG_CMD_I2C

#ifdef CONFIG_CMD_USB
    // usb device init.
    // note: must be after GPIO_init()
	ConfigureUSB();
    Board_initUSB(Board_USBHOST);
#endif //CONFIG_CMD_USB
}
//============================================================================
//============================================================================
// Each Configure Pin and Pad and Function   (board_funcs.h, board_funcs.c)
//----------------------------------------------------------------------------
#include <ti/devices/msp432e4/inc/msp432.h>
#include <ti/devices/msp432e4/driverlib/gpio.h>
#include <ti/devices/msp432e4/driverlib/pin_map.h>
#include <ti/devices/msp432e4/driverlib/sysctl.h>
#include <ti/drivers/I2C.h>
#include <ti/drivers/SPI.h>
#include <ti/drivers/ADCBuf.h>
//----------------------------------------------------------------------------
#ifdef CONFIG_CMD_ADC
void ConfigureADC(void)
{
	// if use ADCs of MSP432E401Y LP
	ADCBuf_init();
	return;
}
#endif //CONFIG_CMD_ADC
#ifdef CONFIG_CMD_I2C
/*
 *  ======== ConfigureI2C ========
 * This function initializes the I2C0,I2C2 and the Associated GPIOs
 */
void ConfigureI2C(void)
{
	// I2C0 (PB2,3)
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_I2C0);
    GPIOPinConfigure(GPIO_PB2_I2C0SCL);
    GPIOPinConfigure(GPIO_PB3_I2C0SDA);
    GPIOPinTypeI2C(GPIO_PORTB_BASE, GPIO_PIN_3);
    GPIOPinTypeI2CSCL(GPIO_PORTB_BASE, GPIO_PIN_2);
    // SCL pull up
    GPIOPadConfigSet(GPIO_PORTB_BASE,GPIO_PIN_2,GPIO_STRENGTH_2MA,GPIO_PIN_TYPE_STD_WPU);

    // I2C2 (PN4,5)
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPION);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_I2C2);
    GPIOPinConfigure(GPIO_PN4_I2C2SDA);
    GPIOPinConfigure(GPIO_PN5_I2C2SCL);
    GPIOPinTypeI2C(GPIO_PORTN_BASE, GPIO_PIN_4);
    GPIOPinTypeI2CSCL(GPIO_PORTN_BASE, GPIO_PIN_5);
    // SCL pull up
    GPIOPadConfigSet(GPIO_PORTN_BASE,GPIO_PIN_5,GPIO_STRENGTH_2MA,GPIO_PIN_TYPE_STD_WPU);

    I2C_init();
}
#endif //CONFIG_CMD_I2C
#ifdef CONFIG_CMD_SPI
/*
 *  ======== ConfigureSPI ========
 * This function initializes the SPI2,SPI3 and the Associated GPIOs
 */
void ConfigureSPI(void)
{
	// SPI2 (PD0,1,3), PD2 as used AIN13(PD2), use PN2,3,PH2 as GPIO-CS
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI2);
    GPIOPinConfigure(GPIO_PD0_SSI2XDAT1);	//GPIO_PD0_SSI2RX);
    GPIOPinConfigure(GPIO_PD1_SSI2XDAT0);	//GPIO_PD1_SSI2TX);
    GPIOPinConfigure(GPIO_PD3_SSI2CLK);
    // PH2(CS Other)
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPION);
    //GPIOPinConfigure(GPIOMSP432E4_PN3 | GPIO_CFG_OUT_STD | GPIO_CFG_OUT_HIGH);

    GPIOPinTypeSSI(GPIO_PORTD_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_3);

	// SPI3 (PQ0,1,2,3)
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOQ);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_SSI3);
    GPIOPinConfigure(GPIO_PQ0_SSI3CLK);
    GPIOPinConfigure(GPIO_PQ3_SSI3XDAT1);	//GPIO_PQ3_SSI3RX);
    GPIOPinConfigure(GPIO_PQ2_SSI3XDAT0);	//GPIO_PQ2_SSI3TX);
    GPIOPinConfigure(GPIO_PQ1_SSI3FSS);
    GPIOPinTypeSSI(GPIO_PORTQ_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3);
    // SS pull up
    GPIOPadConfigSet(GPIO_PORTQ_BASE,GPIO_PIN_1,GPIO_STRENGTH_2MA,GPIO_PIN_TYPE_STD_WPU);

    SPI_init();
}
#endif //CONFIG_CMD_I2C

#ifdef CONFIG_CMD_USB
/*
 *  ======== ConfigureUSB ========
 * This function initializes the USB0 and the Associated GPIOs
 */
void ConfigureUSB(void)
{
	// USB0
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);	// TARGET_ID
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOL);	// USBD_P,N
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);	//
	SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOQ);
	// in Board_initUSB() (MSP_EXP432E401Y_initUSB())
	//Power_setDependency(PowerMSP432E4_PERIPH_USB0);  //Power to USB is turned off in Power_init() and in .cfg file.  With this
	//                                                 //API call, the power is enabled to USB module
	///* Enable the USB peripheral */
	//SysCtlPeripheralEnable(SYSCTL_PERIPH_USB0);
}
#endif //CONFIG_CMD_USB


//============================================================================

