// SPDX-License-Identifier: GPL-2.0+
/*
 * Copyright (C) 2018 BayLibre, SAS
 * Author: Neil Armstrong <narmstrong@baylibre.com>
 */
#include <common.h>
#include <command.h>
#ifdef CONFIG_CMD_ADC

//
// if use RTOS,
// adc configuration is in: MSP_EXP432E401Y.c (+Board.h)
//
// use ADCBuf ...
//

#ifdef TI_RTOS
#include "ti_rtos_io.h"	// in source top directory
#include <Board.h>
#include <assert.h>
#include "board_funcs.h"
#include <ti/drivers/ADCBuf.h>
struct udevice adc_udev[]={
	{ "adc0", 0, Board_ADC, MSP_EXP432E401Y_ADCBUF0, },	// ADC0: for temp sensor Board_ADCBUF0 at chan??
	{ "adc1", 1, Board_ADC, MSP_EXP432E401Y_ADCBUF1, },	// ADC1: for temp sensor Board_ADCBUF0
	// end mark
	{ NULL, -1, Board_None, -1, }
};
//============================
int adc_data_mask(struct udevice *dev, unsigned int *data_mask)
{
	if (dev==NULL || dev->board_dev_type!=Board_ADC) { return -1; }
	if (data_mask==NULL) { return -1; }
	// MSP432E401Y is 12bit ADC
	*data_mask=0x0FFF;
	return 0;
}
int adc_channel_single_shot(const char *name, int channel, unsigned int *data)
{
	struct udevice* dev;
	int ret = uclass_get_device_by_name(UCLASS_ADC, name, &dev);
	if (ret) {
		return -1;
	}

	if (dev==NULL || dev->board_dev_type!=Board_ADC) { return -1; }
	if (data==NULL) { return -1; }

	const int module=dev->board_dev_id; 	//MSP_EXP432E401Y_ADCBUF0;
	const int chan=channel; //MSP_EXP432E401Y_ADCBUF0CHANNELTEMP;
	unsigned long uV=0;
	int val=ti_rtos_adc_single(module,chan,&uV);
	if (val<0) {
		// error
		*data=0;
		return val;
	}
	*data=val;
	printf("%lu.%06luV (%lu.%03lumV)\n",uV/1000000, uV%1000000,uV/1000,uV%1000);
	if (chan==MSP_EXP432E401Y_ADCBUF0CHANNELTEMP) {
		// convert to temp
		int deciTemp=(1475*4096-(75*33*val))/4096;
		printf("%d.%dC\n",deciTemp/10,deciTemp%10);
	}
	return 0;
}
int adc_vdd_value(struct udevice *dev, int *uV)
{
	// maybe return VREFA+(ADCCTL.VREF==1), VDDA(ADCCTL.VREF==0)
	if (dev==NULL || dev->board_dev_type!=Board_ADC) { return -1; }
	if (uV==NULL) { return -1; }
	// now, VREFA+=VDDA=3.3V
	*uV=(3300*1000L);	// 3.3V=3300mV=3300000uV
	return 0;
}
int adc_vss_value(struct udevice *dev, int *uV)
{
	// maybe return VREFA-(ADCCTL.VREF==1), GNDA(ADCCTL.VREF==0)
	if (dev==NULL || dev->board_dev_type!=Board_ADC) { return -1; }
	if (uV==NULL) { return -1; }
	// now, VREFA-=GNDA(as VREF-), GNDA=0V
	*uV=0L;	// uV
	return 0;
}
//============================

//============================
static int do_temparature(cmd_tbl_t *cmdtp, int flag, int argc, char *const argv[])
{
	const int module=MSP_EXP432E401Y_ADCBUF0;
	const int chan=MSP_EXP432E401Y_ADCBUF0CHANNELTEMP;
	unsigned long uV=0;
	int val=ti_rtos_adc_single(module,chan,&uV);
	if (val<0) {
		// error
		return CMD_RET_FAILURE;
	}
	// convert to temp
	int deciTemp=(1475*4096-(75*33*val))/4096;
	printf("%d.%dC\n",deciTemp/10,deciTemp%10);
	//
	return CMD_RET_SUCCESS;
}
U_BOOT_CMD(temp, 4, 1, do_temparature, "temparature of CPU","");
//============================

#else //-TI_RTOS
#include <dm.h>
#endif //-TI_RTOS
#include <adc.h>

static int do_adc_list(cmd_tbl_t *cmdtp, int flag, int argc,
		       char *const argv[])
{
	struct udevice *dev;
	int ret;

	ret = uclass_first_device_err(UCLASS_ADC, &dev);
	if (ret) {
		printf("No available ADC device\n");
		return CMD_RET_FAILURE;
	}

	do {
		printf("- %s\n", dev->name);

		ret = uclass_next_device(&dev);
		if (ret)
			return CMD_RET_FAILURE;
	} while (dev);

	return CMD_RET_SUCCESS;
}

static int do_adc_info(cmd_tbl_t *cmdtp, int flag, int argc,
		       char *const argv[])
{
	struct udevice *dev;
	unsigned int data_mask;
	int ret, vss, vdd;

	if (argc < 2)
		return CMD_RET_USAGE;

	ret = uclass_get_device_by_name(UCLASS_ADC, argv[1], &dev);
	if (ret) {
		printf("Unknown ADC device %s\n", argv[1]);
		return CMD_RET_FAILURE;
	}

#ifdef TI_RTOS
	if (dev==NULL || dev->board_dev_type!=Board_ADC) {
		printf("Unknown ADC device %s\n", argv[1]);
		return CMD_RET_FAILURE;
	}
	printf("ADC Device '%s' :\n", argv[1]);
	const int module=dev->board_dev_id; 	//MSP_EXP432E401Y_ADCBUF0;
	int chan=-1;
	if (argc>=3) {
		chan=simple_strtol(argv[2], NULL, 0);
	}
	ret=ti_rtos_adc_info(module,chan);
	if (ret) {
		return CMD_RET_FAILURE;
	}
#else //-TI_RTOS
	printf("ADC Device '%s' :\n", argv[1]);

	ret = adc_data_mask(dev, &data_mask);
	if (!ret)
		printf("data mask: %x\n", data_mask);

	ret = adc_vdd_value(dev, &vdd);
	if (!ret)
		printf("vdd: %duV\n", vdd);

	ret = adc_vss_value(dev, &vss);
	if (!ret)
		printf("vss: %duV\n", vss);
#endif //TI_RTOS

	return CMD_RET_SUCCESS;
}

static int do_adc_single(cmd_tbl_t *cmdtp, int flag, int argc,
			 char *const argv[])
{
	unsigned int data;
	int ret;

	if (argc < 3)
		return CMD_RET_USAGE;

	ret = adc_channel_single_shot(argv[1], simple_strtol(argv[2], NULL, 0),
				      &data);
	if (ret) {
		printf("Error getting single shot for device %s channel %s\n",
		       argv[1], argv[2]);
		return CMD_RET_FAILURE;
	}

	printf("%u\n", data);

	return CMD_RET_SUCCESS;
}

static cmd_tbl_t cmd_adc_sub[] = {
	U_BOOT_CMD_MKENT(list, 1, 1, do_adc_list, "", ""),
	U_BOOT_CMD_MKENT(info, 2, 1, do_adc_info, "", ""),
	U_BOOT_CMD_MKENT(single, 3, 1, do_adc_single, "", ""),
};

static int do_adc(cmd_tbl_t *cmdtp, int flag, int argc,
		  char *const argv[])
{
	cmd_tbl_t *c;

	if (argc < 2)
		return CMD_RET_USAGE;

	/* Strip off leading 'adc' command argument */
	argc--;
	argv++;

	c = find_cmd_tbl(argv[0], &cmd_adc_sub[0], ARRAY_SIZE(cmd_adc_sub));

	if (c)
		return c->cmd(cmdtp, flag, argc, argv);
	else
		return CMD_RET_USAGE;
}

static char adc_help_text[] =
	"list - list ADC devices\n"
#ifdef TI_RTOS
	"adc info <name> [<channel>]- Get ADC device info\n"
#else //-TI_RTOS
	"adc info <name> - Get ADC device info\n"
#endif //-TI_RTOS
	"adc single <name> <channel> - Get Single data of ADC device channel";

U_BOOT_CMD(adc, 4, 1, do_adc, "ADC sub-system", adc_help_text);

#endif //CONFIG_CMD_ADC
