// UART6 i/o lib WITH TI-RTOS Driver

#include <ti/devices/msp432e4/inc/msp432e411y.h>
#include <sys/types.h>
#include <stdio.h>	// for NULL
#include <stdbool.h>
#include <stdint.h>
#include <ti/devices/msp432e4/driverlib/inc/hw_sysctl.h>
#include <ti/devices/msp432e4/driverlib/inc/hw_uart.h>
//#include "debug.h"
#include <ti/devices/msp432e4/driverlib/interrupt.h>
#include <ti/devices/msp432e4/driverlib/uart.h>
// for pin function
#include <ti/devices/msp432e4/driverlib/gpio.h>
#include <ti/devices/msp432e4/driverlib/pin_map.h>
#include <ti/devices/msp432e4/driverlib/sysctl.h>
#include <ti/drivers/dpl/ClockP.h>
#include <ti/drivers/power/PowerMSP432E4.h>


#include "sleep.h"
#include "uart6.h"
//#ifndef NULL
//# define NULL (0)
//#endif

// for  J7
#ifdef USE_UART6

// appli <-> driver(upper driver IF; HW independ) <-> driverLib(lower driver; IF for HW)
#define USE_UART6_IRQ_RW	// use UART_read instead of UART_readPolling, write too.
#ifdef USE_UART6_IRQ_RW
# define UART6_READ	UART_read
# define UART6_WRITE	UART_write
#else //-USE_UART6_IRQ_RW
# define UART6_READ	UART_readPolling
# define UART6_WRITE	UART_writePolling
#endif //-USE_UART6_IRQ_RW

// use driver  (not lower-driverLib)
#include "Board_ubootcmd.h"
#include <ti/drivers/UART.h>
static UART_Params sUART6Params;
static UART_Handle g_psUART6Handle;

int uart6_init(const int speed)
{
	// Power, SysCtrlPeripheral,PinConfig,PinType
	// is done in MSP_EXP432E401Y.c; see UARTMSP432E4_HWAttrs

	/* Create a UART parameters instance.  The default parameter are 115200
	 * baud, 8 data bits, 1 stop bit and no parity. */
	UART_Params_init(&sUART6Params);

	/* Modify some of the default parameters for this application. */
	sUART6Params.readReturnMode = UART_RETURN_NEWLINE;
	sUART6Params.readEcho = UART_ECHO_OFF;
	//sUART6Params.readDataMode=UART_DATA_TEXT;	// NEVER. cannot work xyModem
	sUART6Params.readDataMode=UART_DATA_BINARY;
	//sUART6Params.readMode=UART_MODE_CALLBACK;
	//sUART6Params.readCallback=uart_recv_callback;
	//sUART6Params.writeDataMode=UART_DATA_BINARY;
	sUART6Params.writeDataMode=UART_DATA_TEXT;	// NEVER. cannot work xyModem

	/* Configure UART0 with the above parameters. */
	g_psUART6Handle = UART_open(Board_UART6, &sUART6Params);
	if (g_psUART6Handle == NULL)
    {
        return -1;
    }
    return 0;
}
int is_uart6_tx_buf_full(void)
{
	// brain-dead TI RTOS. no function.
	return 0;
}
int is_uart6_rx_buf_empty(void)
{
	bool avail=false;
	UART_control(g_psUART6Handle,UART_CMD_ISAVAILABLE,&avail);
	return (! avail);
}
int putc_uart6_tx_buf(const int c)
{
	unsigned char buf[2]={ c, 0 };
	UART6_WRITE(g_psUART6Handle,buf,1);
	return 0;
}
int puts_uart6_tx_buf(const char* s)
{
	for (; *s; ) {
		if (is_uart6_tx_buf_full()) {
			sleep_ms(10);
			continue;
		}
		putc_uart6_tx_buf(*s++);
	}
	return 0;
}
int getc_uart6_rx_buf(void)
{
	unsigned char buf[2]={ 0, 0 };
	int len=UART6_READ(g_psUART6Handle,buf,1);
	if (len!=1) {
		return -1;
	}
	return buf[0];
}

#else //-USE_UART6
int uart6_init(const int speed) { return 0; }
int putc_uart6_tx_buf(const int c) { return 0; }
int puts_uart6_tx_buf(const char* s) { return 0; }

#endif //USE_UART6

